package com.example.nekontakte.core.api;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import com.example.nekontakte.core.configurations.Constants;
import com.example.nekontakte.posts.api.PostDTO;
import com.example.nekontakte.posts.model.PostEntity;

import org.springframework.data.domain.Page;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;

import com.example.nekontakte.posts.service.PostService;
import com.example.nekontakte.users.service.UserService;

@Controller
@RequestMapping(HomeCotroller.URL)
public class HomeCotroller {
    public static final String URL = "/";
    private static final String PAGE_ATTRIBUTE = "page";
    private static final String POSTS_ATTRIBUTE = "posts";
    private static final String POST_ATTRIBUTE = "post";
    private static final String FEED_VIEW = "feed";
    private static final String SUBS_VIEW = "subs";
    private static final String USER_PROFILE_VIEW = "user-profile";

    private final PostService postsService;
    private final UserService userService;

    public HomeCotroller(PostService postsService, UserService userService) {
        this.postsService = postsService;
        this.userService = userService;
    }

    @GetMapping("/feed")
    public String getFeed(@RequestParam(name = PAGE_ATTRIBUTE, defaultValue = "0") int page, Model model) {
        Page<PostEntity> pageEntities = postsService.getAll(page, 4);
        List<PostEntity> posts = pageEntities.getContent();
        List<PostDTO> postDTOs = new ArrayList<PostDTO>();
        for (PostEntity postEntity : posts) {
            PostDTO postDTO = PostDTO.createFromEntity(postEntity);
            postDTOs.add(postDTO);
        }
        model.addAttribute("totalPages", pageEntities.getTotalPages());
        model.addAttribute("currentPage", pageEntities.getNumber());
        model.addAttribute(PAGE_ATTRIBUTE, page);
        model.addAttribute(POSTS_ATTRIBUTE, postDTOs);
        model.addAttribute(POST_ATTRIBUTE, new PostDTO());
        return FEED_VIEW;
    }

    @GetMapping("/subs")
    public String getSubs(Model model) {
        return SUBS_VIEW;
    }

    @GetMapping("/me")
    public String getMyProfile(Model model) {
        return USER_PROFILE_VIEW;
    }

    @GetMapping("/user/{username}")
    public String getUserProfile(Model model,
            @PathVariable(name = "username") String username) {
        return USER_PROFILE_VIEW;
    }
}