package com.example.nekontakte.posts.api;

import com.example.nekontakte.core.configurations.Constants;

import org.apache.coyote.BadRequestException;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import com.example.nekontakte.posts.model.PostEntity;
import com.example.nekontakte.posts.service.PostService;
import com.example.nekontakte.users.service.UserService;

import jakarta.validation.Valid;

@Controller
@RequestMapping(PostsController.URL)
public class PostsController {
    public static final String URL = "/posts";
    private static final String PAGE_ATTRIBUTE = "page";
    private static final String POST_ATTRIBUTE = "post";

    private final PostService postService;
    private final UserService userService;

    public PostsController(PostService postService, UserService userService) {
        this.userService = userService;
        this.postService = postService;
    }

    private PostEntity toEntity(PostDTO dto) {
        PostEntity entity = new PostEntity();
        entity.setId(dto.getId());
        entity.setHtml(dto.getText());
        entity.setImage(dto.getImage());
        entity.setPubDate(dto.getPubDate());
        entity.setUser(userService.get(dto.getUserId()));
        return entity;
    }

    private PostDTO toDTO(PostEntity entity) {
        return PostDTO.createFromEntity(entity);
    }

    @PostMapping("/create/{username}")
    public String create(Model model,
            @PathVariable(name = "username") String username,
            @RequestParam(name = PAGE_ATTRIBUTE, defaultValue = "0") int page,
            @ModelAttribute(name = POST_ATTRIBUTE) @Valid PostDTO postDTO,
            RedirectAttributes redirectAttributes) throws BadRequestException {

        postDTO.setUserId(userService.getByUsername(username).getId());

        postService.create(toEntity(postDTO));

        redirectAttributes.addAttribute(PAGE_ATTRIBUTE, page);

        return Constants.REDIRECT_VIEW + "/feed";
    }

    @PostMapping("/{id}")
    public PostDTO update(@PathVariable(name = "id") Integer id, @RequestBody PostDTO PostDTO) {
        return toDTO(postService.update(id, toEntity(PostDTO)));
    }

    @PostMapping("/delete/{id}")
    public String delete(@PathVariable(name = "id") Integer id,
            @RequestParam(name = PAGE_ATTRIBUTE, defaultValue = "0") int page,
            RedirectAttributes redirectAttributes,
            Model model) {
        postService.delete(id);
        redirectAttributes.addAttribute(PAGE_ATTRIBUTE, page);
        return Constants.REDIRECT_VIEW + "/feed";
    }

}
