package com.example.nekontakte.posts.service;

import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Sort;
import org.springframework.data.domain.Sort.Direction;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Random;

import com.example.nekontakte.core.errors.NotFoundException;
import com.example.nekontakte.posts.model.PostEntity;
import com.example.nekontakte.posts.repository.PostRepository;

import java.time.LocalDateTime;
import java.util.Comparator;
import java.util.List;
import java.util.stream.StreamSupport;

@Service
public class PostService {
    private final PostRepository repository;

    public PostService(PostRepository repository) {
        this.repository = repository;
    }

    public List<PostEntity> getSorted(List<PostEntity> posts) {
        posts.sort(new Comparator<PostEntity>() {
            public int compare(PostEntity o1, PostEntity o2) {
                return o1.getPubDate().compareTo(o2.getPubDate());
            }
        });
        return posts;
    }

    @Transactional(readOnly = true)
    public Page<PostEntity> getAllByUserId(Integer userId, PageRequest pageRequest) {
        return repository.findByUserId(userId, pageRequest);
    }

    @Transactional(readOnly = true)
    public List<PostEntity> getAllByUserId(Integer userId) {
        return repository.findByUserId(userId);
    }

    @Transactional(readOnly = true)
    public Page<PostEntity> getAll(Integer pageNumber, Integer pageSize) {
        PageRequest pageRequest = PageRequest.of(pageNumber, pageSize, Sort.by(Direction.DESC, "pubDate"));
        return repository.findAll(pageRequest);
    }

    @Transactional(readOnly = true)
    public Page<PostEntity> getAll(PageRequest pageRequest) {
        return repository.findAll(pageRequest);
    }

    @Transactional(readOnly = true)
    public List<PostEntity> getAll() {
        return StreamSupport.stream(repository.findAll().spliterator(), false).toList();
    }

    @Transactional(readOnly = true)
    public PostEntity get(Integer id) {
        return repository.findById(id).orElseThrow(() -> new NotFoundException(PostEntity.class, id));
    }

    public PostEntity create(PostEntity entity) throws org.apache.coyote.BadRequestException {
        if (entity.getImage() == null && entity.getHtml() == null) {
            throw new org.apache.coyote.BadRequestException("Image or Html must be not null");
        }
        entity.setPubDate(LocalDateTime.now());
        Random rand = new Random();
        if (rand.nextBoolean()) {
            entity.setImage("https://loremflickr.com/240/320?lock=" + Integer.toString(rand.nextInt(0, 1000000)));
        }
        return repository.save(entity);
    }

    public PostEntity update(Integer id, PostEntity entity) {
        final PostEntity existEntity = get(id);
        existEntity.setUser(entity.getUser());
        existEntity.setPubDate(entity.getPubDate());
        existEntity.setImage(entity.getImage());
        existEntity.setHtml(entity.getHtml());
        repository.save(existEntity);
        return existEntity;
    }

    public PostEntity delete(Integer id) {
        final PostEntity existsentity = get(id);
        repository.delete(existsentity);
        return existsentity;
    }

}
